import 'dart:ui' as ui show window;

import 'package:flutter/widgets.dart';
import 'package:yuro_core/yuro_core.dart';

enum ScreenPixelType { dp, px }

extension ScreenExt on YuroInterface {
  static _Screen _screen = _Screen();

  void initDesign(int uiWidth, int uiHeight, ScreenPixelType type) => _screen = _Screen(uiWidth: uiWidth, uiHeight: uiHeight, type: type);

  double get screenWidth => _screen.width;

  double get screenWidthPx => _screen.widthPx;

  double get screenHeight => _screen.height;

  double get screenHeightPx => _screen.heightPx;

  double get devicePixelRatio => _screen.devicePixelRatio;

  double get textScaleFactor => _screen.textScaleFactor;

  double get statusBarHeight => _screen.statusBarHeight;

  double get bottomBarHeight => _screen.bottomBarHeight;

  num widthConvert(num width) => _screen.getWidth(width);

  num widthPxConvert(num width) => _screen.getWidth(width, type: ScreenPixelType.px);

  num heightConvert(num height) => _screen.getHeight(height);

  num heightPxConvert(num height) => _screen.getHeight(height, type: ScreenPixelType.px);
}

extension ScreenNumExt on num {
  num get w => Yuro.widthConvert(this);

  num get wpx => Yuro.widthPxConvert(this);

  num get h => Yuro.heightConvert(this);

  num get hpx => Yuro.heightPxConvert(this);

  num get wp => Yuro.screenWidth * this;

  num get hp => Yuro.screenHeight * this;
}

class _Screen {
  final int uiWidth;
  final int uiHeight;
  final ScreenPixelType type;

  _Screen({this.uiWidth = 360, this.uiHeight = 640, this.type = ScreenPixelType.dp}) {
    final window = MediaQueryData.fromWindow(ui.window);
    _devicePixelRatio = window.devicePixelRatio;
    _textScaleFactor = window.textScaleFactor;

    _width = window.size.width;
    _height = window.size.height;

    _statusBarHeight = window.padding.top;
    _bottomBarHeight = window.padding.bottom;

    _widthPx = _width * _devicePixelRatio;
    _heightPx = _height * _devicePixelRatio;

    _scaleWidth = (type == ScreenPixelType.dp ? _width : _widthPx) / uiWidth;
    _scaleHeight = (type == ScreenPixelType.dp ? _height : _heightPx) / uiHeight;
  }

  double _devicePixelRatio;
  double _textScaleFactor;

  double _width;
  double _height;

  double _statusBarHeight;
  double _bottomBarHeight;

  double _widthPx;
  double _heightPx;
  double _scaleWidth;
  double _scaleHeight;

  /// 屏幕宽度
  double get width => _width;

  double get widthPx => _widthPx;

  /// 屏幕高度
  double get height => _height;

  double get heightPx => _heightPx;

  /// 屏幕像素密度
  double get devicePixelRatio => _devicePixelRatio;

  /// 字体缩放比例
  double get textScaleFactor => _textScaleFactor;

  /// 状态栏高度
  double get statusBarHeight => _statusBarHeight;

  /// 底部安全高度
  double get bottomBarHeight => _bottomBarHeight;

  /// 宽度方向适配
  num getWidth(num width, {ScreenPixelType type = ScreenPixelType.dp}) =>
      (type == ScreenPixelType.dp ? width : width / _devicePixelRatio) * _scaleWidth;

  /// 高度方向适配
  num getHeight(num height, {ScreenPixelType type = ScreenPixelType.dp}) =>
      (type == ScreenPixelType.dp ? height : height / _devicePixelRatio) * _scaleHeight;

  /// 字体适配
  ///
  /// @param allowScale 字体是否跟随系统缩放, 默认true
  ///
  /// TODO 未验证
  num getSp(num fontSize, {bool allowScale = true}) => allowScale ? fontSize * _scaleWidth : fontSize * _scaleWidth / _textScaleFactor;
}
